/* global google */

'use strict';

var React = require('react');
var ReactDOM = require('react-dom');
var blacklist = require('blacklist');

module.exports = React.createClass({
  displayName: 'exports',

  getDefaultProps: function getDefaultProps() {
    return {
      // WARNING: even if draggable/panControl is enable, props.center will still be authoritative
      draggable: false,
      panControl: false,

      // WARNING: even if rotateControl is enabled, props.heading will still be authoritative
      rotateControl: false,

      // WARNING: if scaleControl is enabled, ... something
      scaleControl: false,

      // WARNING: even if these are enabled, props.zoom will still be authoritative
      disableDoubleClickZoom: true,
      scrollwheel: false,
      zoomControl: false,

      // WARNING: even if enabled, props.mapTypeId will still be authoritative
      mapTypeControl: false,

      // WARNING: be careful with this
      keyboardShortcuts: false,
      streetViewControl: false,

      noClear: true
    };
  },

  getInitialState: function getInitialState() {
    return {};
  },

  componentDidMount: function componentDidMount() {
    this.updateMap(this.props);
  },
  componentWillReceiveProps: function componentWillReceiveProps(newProps) {
    this.updateMap(newProps);
  },

  updateMap: function updateMap(newProps) {
    var _this = this;

    var domNode = ReactDOM.findDOMNode(this);
    var options = blacklist(newProps, 'children', 'pan', 'autofit');
    var map = this.state.map;

    // create new map
    if (!map) {
      map = new google.maps.Map(domNode, options);

      // update existing map
    } else {
        map.setOptions(options);
      }

    // autofit the bounds to the children?
    if (newProps.autofit && newProps.children.length) {
      (function () {
        var bounds = new google.maps.LatLngBounds();

        React.Children.forEach(newProps.children, function (child) {
          var position = child.props.position;
          if (!position) return;

          bounds.extend(new google.maps.LatLng(position.lat, position.lng));
        });

        map.fitBounds(bounds);
      })();
    }

    this.setState({ map: map }, function () {
      if (!_this.props.onUpdate) return;

      _this.props.onUpdate(map);
    });
  },

  getMap: function getMap() {
    return this.state.map;
  },
  getBounds: function getBounds() {
    return this.state.map.getBounds();
  },
  getCenter: function getCenter() {
    return this.state.map.getCenter();
  },
  getHeading: function getHeading() {
    return this.state.map.getHeading();
  },
  getProjection: function getProjection() {
    return this.state.map.getProjection();
  },
  getTilt: function getTilt() {
    return this.state.map.getTilt();
  },
  getZoom: function getZoom() {
    return this.state.map.getZoom();
  },

  render: function render() {
    var map = this.state.map;
    var children = React.Children.map(this.props.children, function (child) {
      return React.cloneElement(child, { map: map });
    });

    return React.createElement(
      'div',
      { style: { height: '100%', width: '100%' } },
      children
    );
  }
});

// expose the marker on the top level export
module.exports.InfoWindow = require('./info-window');
module.exports.Marker = require('./marker');